# ----------------------------------------------------------------------------------------------------------
var.wt <- function(x, w, na.rm = FALSE) {
  if (na.rm) {
    w <- w[i <- !is.na(x)]
    x <- x[i]
  }
  sum.w <- sum(w)
  return((sum(w*x^2) * sum.w - sum(w*x)^2) / (sum.w^2 - sum(w^2)))
}

# ----------------------------------------------------------------------------------------------------------
weighted.t.test <- function(x, y, w.x,w.y, mu, conf.level = 0.95, alternative="two.sided", na.rm=TRUE) {
  if(!missing(conf.level) &
     (length(conf.level) != 1 || !is.finite(conf.level) ||
      conf.level < 0 || conf.level > 1))
    stop("'conf.level' must be a single number between 0 and 1")
  
  if (na.rm) {
    w.x <- w.x[i <- !is.na(x)]
    x <- x[i]
    w.y <- w.y[i <- !is.na(y)]
    y <- y[i]
  }
  
  # to achieve consistent behavior in loops, return NA-structure in case of complete missings
  if (sum(is.na(x)) == length(x)) return(list(estimate=NA, se=NA, conf.int=NA, statistic=NA, df=NA, p.value=NA))
  
  # if only one value is present: this is the best estimate, no significance test provided
  if (sum(!is.na(x)) == 1) {
    warning("Warning weighted.t.test: only one value provided; this value is returned without test of significance!", call.=FALSE)
    return(list(estimate=x[which(!is.na(x))], se=NA, conf.int=NA, statistic=NA, df=NA, p.value=NA))
  }
  
  x.bar <- weighted.mean(x,w.x, na.rm=na.rm)
  y.bar <- weighted.mean(y,w.y,na.rm=na.rm)
  x.var <- var.wt(x,w.x, na.rm=na.rm)
  y.var <- var.wt(y,w.y, na.rm=na.rm)
  diff <- x.bar - y.bar
  x.n <- length(x)
  y.n <- length(y)
  df <- ((x.var/x.n + y.var/y.n)^2)/((x.var/x.n)^2/(x.n - 1) + (y.var/y.n)^2/(y.n-1))
  se <- (sqrt((x.var/x.n) + (y.var/y.n)))
  t.value <- diff/se
  
  
  if (alternative == "less") {
    pval <- pt(t.value, df)
  }
  else if (alternative == "greater") {
    pval <- pt(t.value, df, lower.tail = FALSE)
  }
  else {
    pval <- 2 * pt(-abs(t.value), df)
  }
  
  names(t.value) <- "t"
  return(list(estimate=diff, se=se, statistic=t.value, df=df, p.value=pval))
}

# ----------------------------------------------------------------------------------------------------------
conv.date <- function(x) {
  x <- ifelse(nchar(as.character(x)) == 5,paste("0",x,sep=""),x)
  date <- as.Date(paste(substr(x,1,2),"-",substr(x,3,4),"-",substr(x,5,6),sep=""),format="%m-%d-%y")
  return(date)
}

# ----------------------------------------------------------------------------------------------------------
sweeplm <- function(formula,dat,ind) {
  newd <- model.matrix(formula,model.frame(formula,dat,na.action=na.pass))
  newd <- newd[,-1] # remove the intercept
  newd <- cbind(dat[,as.character(formula[2])],newd) # Add the outcome
  ok <- complete.cases(newd) # drop any observations with missing data
  newd <- newd[ok,]
  ind <- ind[ok]
  newd <- apply(newd,2,function(x) unlist(tapply(x,ind,function(z) z-mean(z,na.rm=TRUE)))) # Sweep function...demean each column based on group assignment
  ind <- sort(ind)
  list(lm(newd[,1]~newd[,-1],as.data.frame(newd)),newd,as.character(ind)) # Regress outcome on design matrix without an intercept
}

# ----------------------------------------------------------------------------------------------------------
robust.se <- function(model, cluster,df.fix=FALSE){
  require(sandwich)
  require(lmtest)
  M <- length(unique(cluster))
  N <- length(cluster)           
  # The df.fix assumes the swept FEs were the clusters
  K <- model$rank + ifelse(df.fix,M,0)
  dfc <- (M/(M - 1)) * ((N - 1)/(N - K))
  uj  <- apply(estfun(model), 2, function(x) tapply(x, cluster, sum));
  rcse.cov <- dfc * sandwich(model, meat. = crossprod(uj)/N)
  rcse.se <- coeftest(model, rcse.cov)
  return(rcse.se)
}

# ----------------------------------------------------------------------------------------------------------
makeTransparent<-function(someColor, alph){
  newColor<-col2rgb(someColor)
  apply(newColor, 2, function(curcoldata){rgb(red=curcoldata[1], green=curcoldata[2],
                                              blue=curcoldata[3],alpha=alph, maxColorValue=255)})
}

# ----------------------------------------------------------------------------------------------------------
interaction_plot_continuous <- function(model, effect = "", moderator = "", varcov="default", minimum="min", maximum="max",colr = "grey",
                                        incr="default", num_points = 10, conf=.95, mean=FALSE, median=FALSE, alph=80, rugplot=T, 
                                        histogram=F, title="Marginal effects plot", xlabel="Value of moderator", 
                                        ylabel="Estimated marginal coefficient",pointsplot = F,plot = F,show_est = F) {
  # Extract Variance Covariance matrix
  if (varcov == "default"){
    covMat = vcov(model)
  }else{
    covMat = varcov
  }
  
  # Extract the data frame of the model
  mod_frame = model.frame(model)
  
  # Get coefficients of variables
  if(effect == "") {
    int.string <- rownames(summary(model)$coefficients)[grepl(":",rownames(summary(model)$coefficients))]
    effect <- substr(int.string,1,regexpr(":",int.string)[1]-1)
  }
  if(moderator == "") {
    int.string <- rownames(summary(model)$coefficients)[grepl(":",rownames(summary(model)$coefficients))]
    moderator <- substr(int.string,regexpr(":",int.string)[1]+1,nchar(int.string))
  }
  interaction <- paste(effect,":",moderator,sep="")
  beta_1 = summary(model)$coefficients[effect,1]
  beta_3 = summary(model)$coefficients[interaction,1]
  
  # Set range of the moderator variable
  # Minimum
  if (minimum == "min"){
    min_val = min(mod_frame[[moderator]])
  }else{
    min_val = minimum
  }
  # Maximum
  if (maximum == "max"){
    max_val = max(mod_frame[[moderator]])
  }else{
    max_val = maximum
  }
  
  # Check if minimum smaller than maximum
  if (min_val > max_val){
    stop("Error: Minimum moderator value greater than maximum value.")
  }
  
  # Determine intervals between values of the moderator
  if (incr == "default"){
    increment = (max_val - min_val)/(num_points - 1)
  }else{
    increment = incr
  }
  
  # Create list of moderator values at which marginal effect is evaluated
  x_2 <- seq(from=min_val, to=max_val, by=increment)
  
  # Compute marginal effects
  delta_1 = beta_1 + beta_3*x_2
  
  # Compute variances
  var_1 = covMat[effect,effect] + (x_2^2)*covMat[interaction, interaction] + 2*x_2*covMat[effect, interaction]
  
  # Standard errors
  se_1 = sqrt(var_1)
  
  # Upper and lower confidence bounds
  z_score = qnorm(1 - ((1 - conf)/2))
  upper_bound = delta_1 + z_score*se_1
  lower_bound = delta_1 - z_score*se_1
  
  # Determine the bounds of the graphing area
  max_y = max(upper_bound)
  min_y = min(lower_bound)
  
  # Make the histogram color
  hist_col = makeTransparent("grey",alph = alph)
  
  # Initialize plotting window
  if(plot) {
    plot(x=c(), y=c(), ylim=c(min_y, max_y), xlim=c(min_val, max_val), xlab=xlabel, ylab=ylabel, main=title)
    
    # Plot estimated effects
    if(!pointsplot) {
      lines(y=delta_1, x=x_2,col = makeTransparent(colr,alph = alph))
      lines(y=upper_bound, x=x_2, lty=2,col = makeTransparent(colr,alph = alph))
      lines(y=lower_bound, x=x_2, lty=2,col = makeTransparent(colr,alph = alph))
    }else{
      points(y = delta_1,x = x_2,col = makeTransparent(colr,alph = alph))
      segments(x_2,upper_bound,x_2,lower_bound,col = makeTransparent(colr,alph = alph))
      segments(x_2 - increment/8,upper_bound,x_2 + increment/8,upper_bound,col = makeTransparent(colr,alph = alph))
      segments(x_2 - increment/8,lower_bound,x_2 + increment/8,lower_bound,col = makeTransparent(colr,alph = alph))
    }
    # Add a dashed horizontal line for zero
    abline(h=0, lty=3)
    
    # Add a vertical line at the mean
    if (mean){
      abline(v = mean(mod_frame[[moderator]]), lty=2, col="red")
    }
    
    # Add a vertical line at the median
    if (median){
      abline(v = median(mod_frame[[moderator]]), lty=3, col="blue")
    }
    
    # Add Rug plot
    if (rugplot){
      rug(mod_frame[[moderator]])
    }
    if (show_est) {
      stars <- ifelse(abs(summary(model)$coefficients[interaction,3]) >2.6,"***",
                      ifelse(abs(summary(model)$coefficients[interaction,3]) > 1.96,"**",
                             ifelse(abs(summary(model)$coefficients[interaction,3]) > 1.65,"*","")))
      text(par('usr')[ 2 ], par('usr')[ 4 ],adj=c(1.05,1.2),
           labels = paste("Interaction: ",round(summary(model)$coefficients[interaction,1],3),stars," (",
                          round(summary(model)$coefficients[interaction,2],3),")",sep=""))
      
    }
    #Add Histogram (Histogram only plots when minimum and maximum are the min/max of the moderator)
    if (histogram & minimum=="min" & maximum=="max"){
      par(new=T)
      hist(mod_frame[[moderator]], axes=F, xlab="", ylab="",main="", border=hist_col, col=hist_col)
    } 
  }
  return(list(delta_1 = delta_1,x_2 = x_2,ub = upper_bound,lb = lower_bound,inc = increment))
}

# ----------------------------------------------------------------------------------------------------------
int_cont_lmer <- function(model, effect, moderator, interaction, varcov="default", minimum="min", maximum="max", incr="default", num_points = 50, conf=.95, mean=FALSE, median=FALSE, alph=80, rugplot=T, histogram=T, title="Marginal effects plot", xlabel="Value of moderator", ylabel="Estimated marginal coefficient"){
  
  # Define a function to make colors transparent
  makeTransparent<-function(someColor, alpha=alph){
    newColor<-col2rgb(someColor)
    apply(newColor, 2, function(curcoldata){rgb(red=curcoldata[1], green=curcoldata[2],
                                                blue=curcoldata[3],alpha=alpha, maxColorValue=255)})
  }
  
  # Extract Variance Covariance matrix
  if (varcov == "default"){
    covMat = vcov(model,useScale=FALSE)
  }else{
    covMat = varcov
  }
  
  # Extract the data frame of the model
  mod_frame = model.frame(model)
  
  # Get coefficients of variables
  beta_1 = fixef(model)[[effect]]
  beta_3 = fixef(model)[[interaction]]
  
  # Set range of the moderator variable
  # Minimum
  if (minimum == "min"){
    min_val = min(mod_frame[[moderator]])
  }else{
    min_val = minimum
  }
  # Maximum
  if (maximum == "max"){
    max_val = max(mod_frame[[moderator]])
  }else{
    max_val = maximum
  }
  
  # Check if minimum smaller than maximum
  if (min_val > max_val){
    stop("Error: Minimum moderator value greater than maximum value.")
  }
  
  # Determine intervals between values of the moderator
  if (incr == "default"){
    increment = (max_val - min_val)/(num_points - 1)
  }else{
    increment = incr
  }
  
  # Create list of moderator values at which marginal effect is evaluated
  x_2 <- seq(from=min_val, to=max_val, by=increment)
  
  # Compute marginal effects
  delta_1 = beta_1 + beta_3*x_2
  
  # Compute variances
  var_1 = covMat[effect,effect] + (x_2^2)*covMat[interaction, interaction] + 2*x_2*covMat[effect, interaction]
  
  # Standard errors
  se_1 = sqrt(var_1)
  
  # Upper and lower confidence bounds
  z_score = qnorm(1 - ((1 - conf)/2))
  upper_bound = delta_1 + z_score*se_1
  lower_bound = delta_1 - z_score*se_1
  
  # Determine the bounds of the graphing area
  max_y = max(upper_bound)
  min_y = min(lower_bound)
  
  # Make the histogram color
  hist_col = makeTransparent("grey")
  
  # Initialize plotting window
  plot(x=c(), y=c(), ylim=c(min_y, max_y), xlim=c(min_val, max_val), xlab=xlabel, ylab=ylabel, main=title)
  
  # Plot estimated effects
  lines(y=delta_1, x=x_2)
  lines(y=upper_bound, x=x_2, lty=2)
  lines(y=lower_bound, x=x_2, lty=2)
  
  # Add a dashed horizontal line for zero
  abline(h=0, lty=3)
  
  # Add a vertical line at the mean
  #if (mean){
  #  abline(v = mean(mod_frame[[moderator]]), lty=2, col="red")
  #}
  
  # Add a vertical line at the median
  #if (median){
  #  abline(v = median(mod_frame[[moderator]]), lty=3, col="blue")
  #}
  
  # Add Rug plot
  if (rugplot){
    rug(mod_frame[[moderator]])
  }
  # Add Histogram (Histogram only plots when minimum and maximum are the min/max of the moderator)
  #if (histogram & minimum=="min" & maximum=="max"){
  # par(new=T)
  # hist(mod_frame[[moderator]], axes=F, xlab="", ylab="",main="", border=hist_col, col=hist_col)
  #}
}


# ----------------------------------------------------------------------------------------------------------
overlay.bounds <- function(model.up,model.lw,effect,moderator.up,moderator.lw,interaction.up,interaction.lw) {
  mod.up = model.frame(model.up)
  min_val <- min(mod.up[[moderator.up]])-10
  max_val <- max(mod.up[[moderator.up]])+10
  increment = (max_val - min_val)/(50 - 1)
  x_2 <- seq(from=min_val, to=max_val, by=increment)
  
  covMat.up = vcov(model.up)
  beta_1.up = fixef(model.up)[[effect]]
  beta_3.up = fixef(model.up)[[interaction.up]]
  
  delta_1.up = beta_1.up + beta_3.up*x_2
  var_1.up = covMat.up[effect,effect] + (x_2^2)*covMat.up[interaction.up,interaction.up] + 2*x_2*covMat.up[effect,interaction.up]
  se_1.up = sqrt(var_1.up)
  
  mod.lw = model.frame(model.lw)
  covMat.lw = vcov(model.lw)
  beta_1.lw = fixef(model.lw)[[effect]]
  beta_3.lw = fixef(model.lw)[[interaction.lw]]
  
  delta_1.lw = beta_1.lw + beta_3.lw*x_2
  var_1.lw = covMat.lw[effect,effect] + (x_2^2)*covMat.lw[interaction.lw,interaction.lw] + 2*x_2*covMat.lw[effect,interaction.lw]
  se_1.lw = sqrt(var_1.lw)
  
  z_score = qnorm(1 - ((1 - .95)/2))
  upper.1 = delta_1.up + z_score*se_1.up
  upper.2 = delta_1.lw + z_score*se_1.lw
  lower.1 = delta_1.lw - z_score*se_1.lw
  lower.2 = delta_1.up - z_score*se_1.up
  
  if(upper.1[1] > upper.2[1]) {
    upper <- upper.1
  } else {
    upper <- upper.2
  }
  if(lower.1[1] < lower.2[1]) {
    lower <- lower.1
  } else {
    lower <- lower.2
  }
  polygon(c(x_2,rev(x_2)),c(upper,rev(lower)),col = rgb(0,0,0,alpha = 60,maxColorValue = 255),border = NA)
  legend("topright",legend=c("Mean Transparency","95% CI","Transparency Bounds"),lty=c(1,2,1),lwd=c(1,1,10),col=c("black","black",rgb(0,0,0,alpha = 60,maxColorValue = 255)),cex=.8)
}


# ----------------------------------------------------------------------------------------------------------
int_bin_lmer <- function(model, effect, moderator, interaction, varcov="default", conf=.95, title="Marginal effects plot", xlabel="Value of moderator", ylabel="Estimated marginal coefficient", factor_labels=c(0,1)){
  
  # Extract Variance Covariance matrix
  if (varcov == "default"){
    covMat = vcov(model,useScale=FALSE)
  }else{
    covMat = varcov
  }
  
  # Extract the data frame of the model
  mod_frame = model.frame(model)
  
  # Get coefficients of variables
  beta_1 = fixef(model)[[effect]]
  beta_3 = fixef(model)[[interaction]]
  
  # Create list of moderator values at which marginal effect is evaluated
  x_2 <- c(0,1)
  
  # Compute marginal effects
  delta_1 = beta_1 + beta_3*x_2
  
  # Compute variances
  var_1 = covMat[effect,effect] + (x_2^2)*covMat[interaction, interaction] + 2*x_2*covMat[effect, interaction]
  
  # Standard errors
  se_1 = sqrt(var_1)
  
  # Upper and lower confidence bounds
  z_score = qnorm(1 - ((1 - conf)/2))
  upper_bound = delta_1 + z_score*se_1
  lower_bound = delta_1 - z_score*se_1
  
  # Determine the bounds of the graphing area
  max_y = max(upper_bound)
  min_y = min(lower_bound)
  
  # Initialize plotting window
  plot(x=c(), y=c(), ylim=c(min_y, max_y), xlim=c(-.5, 1.5), xlab=xlabel, ylab=ylabel, main=title, xaxt="n")
  
  # Plot points of estimated effects
  points(x=x_2, y=delta_1, pch=16)
  
  # Plot lines of confidence intervals
  lines(x=c(x_2[1], x_2[1]), y=c(upper_bound[1], lower_bound[1]), lty=1)
  points(x=c(x_2[1], x_2[1]), y=c(upper_bound[1], lower_bound[1]), pch=c(25,24), bg="black")
  lines(x=c(x_2[2], x_2[2]), y=c(upper_bound[2], lower_bound[2]), lty=1)
  points(x=c(x_2[2], x_2[2]), y=c(upper_bound[2], lower_bound[2]), pch=c(25,24), bg="black")
  
  # Label the axis
  axis(side=1, at=c(0,1), labels=factor_labels)
  
  # Add a dashed horizontal line for zero
  abline(h=0, lty=3)
  
}


# ----------------------------------------------------------------------------------------------------------
genConfound <- function(alpha, delta,dep,treat) {
  e <- rnorm(nrow(test.dat),0,1)
  U <- alpha * test.dat[,paste(dep,collapse="")] + delta * test.dat[,paste(treat,collapse="")] + e
  return(U)
}


# ----------------------------------------------------------------------------------------------------------
LEGEND <- function (x, y = NULL, legend, fill = NULL, 
                    col = par("col"), pt.col=col, line.col=col,
                    border = "black", lty, lwd, pch, angle = 45, density = NULL,
                    bty = "o", bg = par("bg"), box.lwd = par("lwd"), box.lty = par("lty"),
                    box.col = par("fg"), pt.bg = NA, cex = 1, pt.cex = cex, pt.lwd = lwd,
                    xjust = 0, yjust = 1, x.intersp = 1, y.intersp = 1, adj = c(0,
                                                                                0.5), text.width = NULL, text.col = par("col"), text.font = NULL,
                    merge = do.lines && has.pch, trace = FALSE, plot = TRUE,
                    ncol = 1, horiz = FALSE, title = NULL, inset = 0, xpd, title.col = text.col,
                    title.adj = 0.5, seg.len = 2)
{
  if (missing(legend) && !missing(y) && (is.character(y) ||
                                         is.expression(y))) {
    legend <- y
    y <- NULL
  }
  mfill <- !missing(fill) || !missing(density)
  if (!missing(xpd)) {
    op <- par("xpd")
    on.exit(par(xpd = op))
    par(xpd = xpd)
  }
  title <- as.graphicsAnnot(title)
  if (length(title) > 1)
    stop("invalid 'title'")
  legend <- as.graphicsAnnot(legend)
  n.leg <- if (is.call(legend))
    1
  else length(legend)
  if (n.leg == 0)
    stop("'legend' is of length 0")
  auto <- if (is.character(x))
    match.arg(x, c("bottomright", "bottom", "bottomleft",
                   "left", "topleft", "top", "topright", "right", "center"))
  else NA
  if (is.na(auto)) {
    xy <- xy.coords(x, y)
    x <- xy$x
    y <- xy$y
    nx <- length(x)
    if (nx < 1 || nx > 2)
      stop("invalid coordinate lengths")
  }
  else nx <- 0
  xlog <- par("xlog")
  ylog <- par("ylog")
  rect2 <- function(left, top, dx, dy, density = NULL, angle,
                    ...) {
    r <- left + dx
    if (xlog) {
      left <- 10^left
      r <- 10^r
    }
    b <- top - dy
    if (ylog) {
      top <- 10^top
      b <- 10^b
    }
    rect(left, top, r, b, angle = angle, density = density,
         ...)
  }
  segments2 <- function(x1, y1, dx, dy, ...) {
    x2 <- x1 + dx
    if (xlog) {
      x1 <- 10^x1
      x2 <- 10^x2
    }
    y2 <- y1 + dy
    if (ylog) {
      y1 <- 10^y1
      y2 <- 10^y2
    }
    segments(x1, y1, x2, y2, ...)
  }
  points2 <- function(x, y, ...) {
    if (xlog)
      x <- 10^x
    if (ylog)
      y <- 10^y
    points(x, y, ...)
  }
  text2 <- function(x, y, ...) {
    if (xlog)
      x <- 10^x
    if (ylog)
      y <- 10^y
    text(x, y, ...)
  }
  if (trace)
    catn <- function(...) do.call("cat", c(lapply(list(...),
                                                  formatC), list("\n")))
  cin <- par("cin")
  Cex <- cex * par("cex")
  if (is.null(text.width))
    text.width <- max(abs(strwidth(legend, units = "user",
                                   cex = cex, font = text.font)))
  else if (!is.numeric(text.width) || text.width < 0)
    stop("'text.width' must be numeric, >= 0")
  xc <- Cex * xinch(cin[1L], warn.log = FALSE)
  yc <- Cex * yinch(cin[2L], warn.log = FALSE)
  if (xc < 0)
    text.width <- -text.width
  xchar <- xc
  xextra <- 0
  yextra <- yc * (y.intersp - 1)
  ymax <- yc * max(1, strheight(legend, units = "user", cex = cex)/yc)
  ychar <- yextra + ymax
  if (trace)
    catn("  xchar=", xchar, "; (yextra,ychar)=", c(yextra,
                                                   ychar))
  if (mfill) {
    xbox <- xc * 0.8
    ybox <- yc * 0.5
    dx.fill <- xbox
  }
  do.lines <- (!missing(lty) && (is.character(lty) || any(lty >
                                                            0))) || !missing(lwd)
  n.legpercol <- if (horiz) {
    if (ncol != 1)
      warning(gettextf("horizontal specification overrides: Number of columns := %d",
                       n.leg), domain = NA)
    ncol <- n.leg
    1
  }
  else ceiling(n.leg/ncol)
  has.pch <- !missing(pch) && length(pch) > 0
  if (do.lines) {
    x.off <- if (merge)
      -0.7
    else 0
  }
  else if (merge)
    warning("'merge = TRUE' has no effect when no line segments are drawn")
  if (has.pch) {
    if (is.character(pch) && !is.na(pch[1L]) && nchar(pch[1L],
                                                      type = "c") > 1) {
      if (length(pch) > 1)
        warning("not using pch[2..] since pch[1L] has multiple chars")
      np <- nchar(pch[1L], type = "c")
      pch <- substr(rep.int(pch[1L], np), 1L:np, 1L:np)
    }
    if (!is.character(pch))
      pch <- as.integer(pch)
  }
  if (is.na(auto)) {
    if (xlog)
      x <- log10(x)
    if (ylog)
      y <- log10(y)
  }
  if (nx == 2) {
    x <- sort(x)
    y <- sort(y)
    left <- x[1L]
    top <- y[2L]
    w <- diff(x)
    h <- diff(y)
    w0 <- w/ncol
    x <- mean(x)
    y <- mean(y)
    if (missing(xjust))
      xjust <- 0.5
    if (missing(yjust))
      yjust <- 0.5
  }
  else {
    h <- (n.legpercol + (!is.null(title))) * ychar + yc
    w0 <- text.width + (x.intersp + 1) * xchar
    if (mfill)
      w0 <- w0 + dx.fill
    if (do.lines)
      w0 <- w0 + (seg.len + x.off) * xchar
    w <- ncol * w0 + 0.5 * xchar
    if (!is.null(title) && (abs(tw <- strwidth(title, units = "user",
                                               cex = cex) + 0.5 * xchar)) > abs(w)) {
      xextra <- (tw - w)/2
      w <- tw
    }
    if (is.na(auto)) {
      left <- x - xjust * w
      top <- y + (1 - yjust) * h
    }
    else {
      usr <- par("usr")
      inset <- rep_len(inset, 2)
      insetx <- inset[1L] * (usr[2L] - usr[1L])
      left <- switch(auto, bottomright = , topright = ,
                     right = usr[2L] - w - insetx, bottomleft = ,
                     left = , topleft = usr[1L] + insetx, bottom = ,
                     top = , center = (usr[1L] + usr[2L] - w)/2)
      insety <- inset[2L] * (usr[4L] - usr[3L])
      top <- switch(auto, bottomright = , bottom = , bottomleft = usr[3L] +
                      h + insety, topleft = , top = , topright = usr[4L] -
                      insety, left = , right = , center = (usr[3L] +
                                                             usr[4L] + h)/2)
    }
  }
  if (plot && bty != "n") {
    if (trace)
      catn("  rect2(", left, ",", top, ", w=", w, ", h=",
           h, ", ...)", sep = "")
    rect2(left, top, dx = w, dy = h, col = bg, density = NULL,
          lwd = box.lwd, lty = box.lty, border = box.col)
  }
  xt <- left + xchar + xextra + (w0 * rep.int(0:(ncol - 1),
                                              rep.int(n.legpercol, ncol)))[1L:n.leg]
  yt <- top - 0.5 * yextra - ymax - (rep.int(1L:n.legpercol,
                                             ncol)[1L:n.leg] - 1 + (!is.null(title))) * ychar
  if (mfill) {
    if (plot) {
      if (!is.null(fill))
        fill <- rep_len(fill, n.leg)
      rect2(left = xt, top = yt + ybox/2, dx = xbox, dy = ybox,
            col = fill, density = density, angle = angle,
            border = border)
    }
    xt <- xt + dx.fill
  }
  if (plot && (has.pch || do.lines)) {
    pt.COL <- rep_len(pt.col, n.leg)
    line.COL <- rep_len(line.col, n.leg)
  }
  if (missing(lwd) || is.null(lwd))
    lwd <- par("lwd")
  if (do.lines) {
    if (missing(lty) || is.null(lty))
      lty <- 1
    lty <- rep_len(lty, n.leg)
    lwd <- rep_len(lwd, n.leg)
    ok.l <- !is.na(lty) & (is.character(lty) | lty > 0) &
      !is.na(lwd)
    if (trace)
      catn("  segments2(", xt[ok.l] + x.off * xchar, ",",
           yt[ok.l], ", dx=", seg.len * xchar, ", dy=0, ...)")
    if (plot)
      segments2(xt[ok.l] + x.off * xchar, yt[ok.l], dx = seg.len *
                  xchar, dy = 0, lty = lty[ok.l], lwd = lwd[ok.l],
                col = line.COL[ok.l])
    xt <- xt + (seg.len + x.off) * xchar
  }
  if (has.pch) {
    pch <- rep_len(pch, n.leg)
    pt.bg <- rep_len(pt.bg, n.leg)
    pt.cex <- rep_len(pt.cex, n.leg)
    pt.lwd <- rep_len(pt.lwd, n.leg)
    ok <- !is.na(pch)
    if (!is.character(pch)) {
      ok <- ok & (pch >= 0 | pch <= -32)
    }
    else {
      ok <- ok & nzchar(pch)
    }
    x1 <- (if (merge && do.lines)
      xt - (seg.len/2) * xchar
      else xt)[ok]
    y1 <- yt[ok]
    if (trace)
      catn("  points2(", x1, ",", y1, ", pch=", pch[ok],
           ", ...)")
    if (plot)
      points2(x1, y1, pch = pch[ok], col = pt.COL[ok], cex = pt.cex[ok],
              bg = pt.bg[ok], lwd = pt.lwd[ok])
  }
  xt <- xt + x.intersp * xchar
  if (plot) {
    if (!is.null(title))
      text2(left + w * title.adj, top - ymax, labels = title,
            adj = c(title.adj, 0), cex = cex, col = title.col)
    text2(xt, yt, labels = legend, adj = adj, cex = cex,
          col = text.col, font = text.font)
  }
  invisible(list(rect = list(w = w, h = h, left = left, top = top),
                 text = list(x = xt, y = yt)))
}


# ----------------------------------------------------------------------------------------------------------
custom_mfx <- function(model, effect, moderator, interaction, varcov="default", minimum="min", maximum="max", incr="default", num_points = 50, conf=.95, mean=FALSE, median=FALSE, alph=80, rugplot=T, histogram=T, title="Marginal effects plot", xlabel="Value of moderator", ylabel="Estimated marginal coefficient"){
  # Extract Variance Covariance matrix
  if (varcov == "default"){
    covMat = vcov(model,useScale=FALSE)
  }else{
    covMat = varcov
  }
  
  # Extract the data frame of the model
  mod_frame = model.frame(model)
  
  # Get coefficients of variables
  beta_1 = fixef(model)[[effect]]
  beta_3 = fixef(model)[[interaction]]
  
  # Set range of the moderator variable
  # Minimum
  if (minimum == "min"){
    min_val = min(mod_frame[[moderator]])
  }else{
    min_val = minimum
  }
  # Maximum
  if (maximum == "max"){
    max_val = max(mod_frame[[moderator]])
  }else{
    max_val = maximum
  }
  
  # Check if minimum smaller than maximum
  if (min_val > max_val){
    stop("Error: Minimum moderator value greater than maximum value.")
  }
  
  # Determine intervals between values of the moderator
  if (incr == "default"){
    increment = (max_val - min_val)/(num_points - 1)
  }else{
    increment = incr
  }
  
  # Create list of moderator values at which marginal effect is evaluated
  x_2 <- seq(from=min_val, to=max_val, by=increment)
  
  # Compute marginal effects
  delta_1 = beta_1 + beta_3*x_2
  
  # Compute variances
  var_1 = covMat[effect,effect] + (x_2^2)*covMat[interaction, interaction] + 2*x_2*covMat[effect, interaction]
  
  # Standard errors
  se_1 = sqrt(var_1)
  
  # Upper and lower confidence bounds
  z_score = qnorm(1 - ((1 - conf)/2))
  upper_bound = delta_1 + z_score*se_1
  lower_bound = delta_1 - z_score*se_1
  
  # Determine the bounds of the graphing area
  max_y = max(upper_bound)
  min_y = min(lower_bound)
  
  results <- list("min_y" = min_y,"max_y" = max_y,"min_val"=min_val,"max_val"=max_val,"delta_1"=delta_1,"x_2"=x_2,"upper_bound"=upper_bound,"lower_bound"=lower_bound,"rug"=mod_frame[[moderator]])
  return(results)
}



# ----------------------------------------------------------------------------------------------------------
adf.test2 <- function (x, alternative = c("stationary", "explosive"), k = trunc((length(x) - 1)^(1/3))) {
  if ((NCOL(x) > 1) || is.data.frame(x)) 
    stop("x is not a vector or univariate time series")
  if (any(is.na(x))) 
    stop("NAs in x")
  if (k < 0) 
    stop("k negative")
  alternative <- match.arg(alternative)
  DNAME <- deparse(substitute(x))
  k <- k + 1
  x <- as.vector(x, mode = "double")
  y <- diff(x)
  n <- length(y)
  z <- embed(y, k)
  yt <- z[, 1]
  xt1 <- x[k:n]
  tt <- k:n
  if (k > 1) {
    yt1 <- z[, 2:k]
    res <- lm(yt ~ xt1 + 1 + tt + yt1)
  }
  else res <- lm(yt ~ xt1 + 1 + tt)
  res.sum <- summary(res)
  STAT <- res.sum$coefficients[2, 1]/res.sum$coefficients[2,2]
  table <- cbind(c(4.38, 4.15, 4.04, 3.99, 3.98, 3.96), 
                 c(3.95,3.8, 3.73, 3.69, 3.68, 3.66), 
                 c(3.6, 3.5, 3.45, 3.43,3.42, 3.41), 
                 c(3.24, 3.18, 3.15, 3.13, 3.13, 3.12), 
                 c(1.14,1.19, 1.22, 1.23, 1.24, 1.25), 
                 c(0.8, 0.87, 0.9, 0.92,0.93, 0.94), 
                 c(0.5, 0.58, 0.62, 0.64, 0.65, 0.66), 
                 c(0.15, 0.24, 0.28, 0.31, 0.32, 0.33))
  table <- -table
  tablen <- dim(table)[2]
  tableT <- c(25, 50, 100, 250, 500, 1e+05)
  tablep <- c(0.01, 0.025, 0.05, 0.1, 0.9, 0.95, 0.975, 0.99)
  tableipl <- numeric(tablen)
  for (i in (1:tablen)) tableipl[i] <- approx(tableT, table[,i], n, rule = 2)$y
  CRIT <- tableipl[1:3]
  interpol <- approx(tableipl, tablep, STAT, rule = 2)$y
  if (is.na(approx(tableipl, tablep, STAT, rule = 1)$y)) 
    if (interpol == min(tablep)) 
      warning("p-value smaller than printed p-value")
  else warning("p-value greater than printed p-value")
  if (alternative == "stationary") 
    PVAL <- interpol
  else if (alternative == "explosive") 
    PVAL <- 1 - interpol
  else stop("irregular alternative")
  PARAMETER <- k - 1
  METHOD <- "Augmented Dickey-Fuller Test"
  names(STAT) <- "Dickey-Fuller"
  names(PARAMETER) <- "Lag order"
  names(CRIT) <- c("1pct","5pct","10pct")
  structure(list(statistic = STAT, parameter = PARAMETER, alternative = alternative, 
                 p.value = PVAL, method = METHOD, data.name = DNAME,cval = CRIT),class = "htest")
}

# ----------------------------------------------------------------------------------------------------------
cointegration<-function(x,y,lag,crit.val)
{
  vals<-data.frame(x,y)
  beta<-coef(lm(vals[,2]~vals[,1]+0,data=vals))[1]
  temp <- adf.test2(vals[,2]-beta*vals[,1], alternative="s", k=lag)
  crit.stat <- temp$cval[crit.val]
  test.stat <- temp$statistic
  results <- cbind(crit.stat,test.stat)
  return(results)
}
